@@ -13,6 +13,7 @@ from invar.core.lambda_helpers import (
     extract_lambda_params,
     extract_used_names,
     find_lambda,
+    generate_lambda_fix,
 )
 from invar.core.models import FileInfo, RuleConfig, Severity, SymbolKind, Violation
 from invar.core.suggestions import format_suggestion_for_violation
@@ -411,8 +412,11 @@ def check_param_mismatch(file_info: FileInfo, config: RuleConfig) -> list[Violat
         >>> from invar.core.models import FileInfo, Symbol, SymbolKind, Contract, RuleConfig
         >>> c = Contract(kind="pre", expression="lambda x: x > 0", line=1)
         >>> s = Symbol(name="f", kind=SymbolKind.FUNCTION, line=1, end_line=5, signature="(x: int, y: int) -> int", contracts=[c])
-        >>> check_param_mismatch(FileInfo(path="c.py", lines=10, symbols=[s], is_core=True), RuleConfig())[0].rule
+        >>> v = check_param_mismatch(FileInfo(path="c.py", lines=10, symbols=[s], is_core=True), RuleConfig())[0]
+        >>> v.rule
         'param_mismatch'
+        >>> v.suggestion  # DX-01: Now includes fix template
+        'Fix: @pre(lambda x, y: <condition>)'
     """
     violations: list[Violation] = []
     if not file_info.is_core:
@@ -427,6 +431,8 @@ def check_param_mismatch(file_info: FileInfo, config: RuleConfig) -> list[Violat
             mismatch, desc = has_param_mismatch(contract.expression, symbol.signature)
             if mismatch:
                 kind = "Method" if symbol.kind == SymbolKind.METHOD else "Function"
+                # DX-01: Generate copy-pastable lambda fix template
+                fix_template = generate_lambda_fix(symbol.signature)
                 violations.append(
                     Violation(
                         rule="param_mismatch",
@@ -434,7 +440,7 @@ def check_param_mismatch(file_info: FileInfo, config: RuleConfig) -> list[Violat
                         file=file_info.path,
                         line=contract.line,
                         message=f"{kind} '{symbol.name}' @pre {desc}",
-                        suggestion="Lambda must include ALL function parameters",
+                        suggestion=f"Fix: {fix_template}",
                     )
                 )
     return violations
